'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _entries = require('lodash/entries');

var _entries2 = _interopRequireDefault(_entries);

var _JSONSchemaRenderer = require('./JSONSchemaRenderer.scss');

var _JSONSchemaRenderer2 = _interopRequireDefault(_JSONSchemaRenderer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var className = 'json-schema-renderer';

/**
 * UnkownTypeException
 *
 * @param {string} type - The unkown type
 * @returns {Object} An UnkownTypeException
 */
function UnkownTypeException(type) {
	this.name = 'UnkownTypeException';
	this.message = 'Unkown type: ' + type;
}

/**
 * InvalidSchemaException
 *
 * @returns {Object} An InvalidSchemaException
 */
function InvalidSchemaException() {
	this.name = 'InvalidSchemaException';
	this.message = 'Invalid Schema';
}

/**
 * textRenderer
 *
 * @param key
 * @param title
 * @param text
 * @returns {string} - HTML markup for a text component
 */
function textRenderer(key, title, text) {
	return _react2.default.createElement(
		'div',
		{ className: (0, _classnames2.default)('text-renderer', 'text-renderer-' + key), key: key },
		_react2.default.createElement(
			'dt',
			null,
			title || key
		),
		_react2.default.createElement(
			'dd',
			null,
			text
		)
	);
}

function booleanRenderer(key, title, value) {
	return _react2.default.createElement(
		'div',
		{ className: (0, _classnames2.default)('boolean-renderer', 'boolean-renderer-' + key), key: key },
		_react2.default.createElement(
			'dt',
			null,
			title || key
		),
		_react2.default.createElement(
			'dd',
			null,
			value.toString()
		)
	);
}

/**
 * arrayRenderer
 *
 * @param key
 * @param title
 * @param items
 * @returns {string} - HTML markup for an array component
 */
function arrayRenderer(key, title, items) {
	return _react2.default.createElement(
		'div',
		{ className: (0, _classnames2.default)(_JSONSchemaRenderer2.default.array, 'array-renderer-' + key), key: key },
		_react2.default.createElement(
			'dt',
			null,
			title || key
		),
		items.map(function (val, i) {
			return _react2.default.createElement(
				'dd',
				{ key: 'key-' + i },
				val
			);
		})
	);
}

var registry = {
	string: textRenderer,
	integer: textRenderer,
	boolean: booleanRenderer,
	array: arrayRenderer,
	object: objectRenderer };

/**
 * typeResolver
 *
 * @param schema - The JSONSchema of the data being rendered
 * @throws {UnkownTypeException} Type must be part of the registry
 * @returns {Function} resolver receiving data from a map operation
 */
function typeResolver(schema) {
	return function resolver(e) {
		if (!schema[e[0]]) {
			return null;
		}
		var type = schema[e[0]].type;
		var title = schema[e[0]].title;

		var renderer = registry[type];
		if (!renderer) {
			throw new UnkownTypeException(type);
		}

		return renderer(e[0], title, e[1], schema);
	};
}

/**
 * objectRenderer
 *
 * @param key
 * @param title
 * @param properties
 * @param schema
 * @returns {string} - HTML markup for an object component
 */
function objectRenderer(key, title, properties, schema) {
	var flattenProperties = (0, _entries2.default)(properties);
	var elements = flattenProperties.map(typeResolver(schema[key].properties));
	return _react2.default.createElement(
		'div',
		{ className: (0, _classnames2.default)(_JSONSchemaRenderer2.default.object, 'object-renderer-' + key), key: key },
		_react2.default.createElement(
			'h2',
			null,
			title || key
		),
		_react2.default.createElement(
			'div',
			null,
			elements
		)
	);
}

/**
 * orderProperties sorts properties based on uiSchema ui:order array
 *
 * @param order
 * @param properties
 * @returns {Array}
 */
function orderProperties(order, properties) {
	if (!order) {
		return properties;
	}
	return properties.sort(function (a, b) {
		var aIndex = order.indexOf(a[0]);
		var bIndex = order.indexOf(b[0]);
		if (aIndex < 0) {
			return 1;
		}
		if (bIndex < 0) {
			return -1;
		}
		return aIndex - bIndex;
	});
}

/**
 * JSONSchemaRenderer renders elements based on a JSONSchema and data
 *
 * @throws {InvalidSchemaException} schema must contain a jsonSchema and
 * properties
 * @returns {string} - HTML markup for the component
 */
function JSONSchemaRenderer(props) {
	if (!props.schema.jsonSchema || !props.schema.properties) {
		throw new InvalidSchemaException();
	}
	var properties = (0, _entries2.default)(props.schema.properties);
	if (props.schema.uiSchema) {
		properties = orderProperties(props.schema.uiSchema['ui:order'], properties);
	}
	var elements = properties.map(typeResolver(props.schema.jsonSchema.properties));
	return _react2.default.createElement(
		'dl',
		{ className: (0, _classnames2.default)(_JSONSchemaRenderer2.default[className], 'json-schema-renderer') },
		elements
	);
}

JSONSchemaRenderer.propTypes = {
	schema: _react.PropTypes.shape({
		jsonSchema: _react.PropTypes.object.isRequired,
		uiSchema: _react.PropTypes.shape({
			'ui:order': _react.PropTypes.array
		}),
		properties: _react.PropTypes.object.isRequired
	}).isRequired
};

JSONSchemaRenderer.InvalidSchemaException = InvalidSchemaException;
JSONSchemaRenderer.UnkownTypeException = UnkownTypeException;

exports.default = JSONSchemaRenderer;